function FiscalFunctions(FiscalFunctionsService, ScreenService, OperatorRepository, UtilitiesService) {

var self = this;

	this.radioChangeType = function (opcao, field) {
		field.isVisible = (opcao.value() === 'T') ? false : true;
		field.clearValue();
    };

	this.selectCpfOrCnpjMask = function(field) {
		var cpfCnpjValue = field.value();
		var strMask = cpfCnpjValue.replace(/\D/g,"");

		if (cpfCnpjValue.length <= 14) {
			strMask = strMask.replace(/(\d{3})(\d)/,"$1.$2");
			strMask = strMask.replace(/(\d{3})(\d)/,"$1.$2");
			strMask = strMask.replace(/(\d{3})(\d{1,2})$/,"$1-$2");
		} else if (cpfCnpjValue.length > 14 && cpfCnpjValue.length <= 18){
			strMask = strMask.replace(/(\d{2})(\d)/,"$1.$2");
			strMask = strMask.replace(/(\d{3})(\d)/,"$1.$2");
			strMask = strMask.replace(/(\d{3})(\d)/,"$1/$2");
			strMask = strMask.replace(/(\d{4})(\d{1,2})$/,"$1-$2");
		} else {
			strMask = cpfCnpjValue.substr(0, 17);
		}

		$("#cpfCnpj").val(strMask);
	};

	this.isValidCpfCnpj = function(field) {
		if (UtilitiesService.isValidCPForCNPJ(field.value().replace(/[^0-9 ]/g, ""))) {
			return true;
		} 

		return false;
	};

	this.isValidMonthAndYear = function(field) {
		if (field.value()) {
	        var atualDate = new Date();
	        var atualMonth = atualDate.getMonth() + 1;
			var atualYear = atualDate.getFullYear();

	        var date = (field.value()).split('/');
	        var month = parseInt(date[0]);
	        var year = parseInt(date[1]);

	        var invalidDate = (year >= atualYear && month > atualMonth) ? true : false;

	        if (UtilitiesService.validateMonthAndYear(field.value()) && year > 1980 && !invalidDate) {
	            return true;
			} 
		}
		
        return false;

	};

	this.validateDate = function(field) {
		if (field.value()) {
			var atualDate = new Date();

			var atualDay = atualDate.getDate();
	        var atualMonth = atualDate.getMonth() + 1;
			var atualYear = atualDate.getFullYear();

			var date = (field.value()).split('/');

			var day = parseInt(date[0]);
			var month = parseInt(date[1]);
        	var year = parseInt(date[2]);

        	var invalidDay = (year >= atualYear && month >= atualMonth && day > atualDay) ? true : false;
        	var invalidMonth = (year >= atualYear && month > atualMonth) ? true : false;

			if(UtilitiesService.validateDate(field.value()) && year > 1980 && !invalidDay && !invalidMonth) {
				return true;
			}
		}

		return false;
	};

	this.validateIntervalDates = function (initialDate, finalDate) {
		var initialDateSplitted = (initialDate).split('/');
		var finalDateSplitted = (finalDate).split('/');

		var dids = parseInt(initialDateSplitted[0]);
		var mids = parseInt(initialDateSplitted[1]);
		var yids = parseInt(initialDateSplitted[2]);

		var dfds = parseInt(finalDateSplitted[0]);
		var mfds = parseInt(finalDateSplitted[1]);
		var yfds = parseInt(finalDateSplitted[2]);


		var invalidYear = (yids > yfds) ? true : false;
		var invalideMonth = (yids >= yfds && mids > mfds ) ? true : false;
		var invalidDay = (yids >=  yfds && mids >= mfds && dids > dfds) ? true : false;

		if (!invalidYear && !invalideMonth && !invalidDay) {
			return true;
		}

		return false;
	};

	this.generateReportVendasCPFCNPJ = function (widget) {
		OperatorRepository.findOne().then(function(operatorData){ 
			var isValidCPFAndCNPJ;
			var cpfCnpj;
			
			if (widget.currentRow.radioChangeType == 'T') {
				isValidCPFAndCNPJ = true;
				cpfCnpj = 'T';
			} else {
				isValidCPFAndCNPJ = self.isValidCpfCnpj(widget.getField('cpfCnpj'));
        		cpfCnpj = widget.getField('cpfCnpj').value();
			}

        	var isValidMonthAndYear = self.isValidMonthAndYear(widget.getField('monthAndYearSales'));
        	var periodo = widget.getField('monthAndYearSales').value();

        	if (!widget.isValid()){
	            ScreenService.showMessage("É necessário preencher todos os campos para gerar o relatório.");
	        } else if(isValidCPFAndCNPJ && isValidMonthAndYear) {

				ScreenService.showLoader();
				FiscalFunctionsService.vendasCPFCNPJ(operatorData.CDFILIAL, cpfCnpj, periodo).then(function(response) {
					ScreenService.hideLoader();
					if(!_.isEmpty(response)) {
						ScreenService.showMessage(response[0].message);
						if(!response[0].error && response[0].data === 0) {
							self.handleBack(widget);
						}
					} else {
						ScreenService.showMessage("Erro desconhecido ao tentar gerar o relatório.<br> Tente novamente.");
					}
				});
				ScreenService.hideLoader();

			} else {
				var errMsg = "";
	        	errMsg += !isValidCPFAndCNPJ ? 'CPF/CNPF inválido.<br>' : '';
	        	errMsg += !isValidMonthAndYear ? 'Mês e/ou ano inválido(s) para geração do relatório.<br>': '';


				ScreenService.showMessage(errMsg + 'Verifique os campos preenchidos e tente novamente.');
			}
			
		});
	};

	this.generateReportRegistroPaf = function (widget) {
		OperatorRepository.findOne().then(function(operatorData){



			var isValidDataInicio = self.validateDate(widget.getField('initialDate'));
			var isValidDataFim = self.validateDate(widget.getField('finalDate'));
        	var dataInicio = widget.getField('initialDate').value();
        	var dataFim = widget.getField('finalDate').value();
			var isValidIntervalDates = self.validateIntervalDates(dataInicio, dataFim);


			if (!widget.isValid()){
	            ScreenService.showMessage("É necessário preencher todos os campos para gerar o relatório.");
	        } else if(isValidDataInicio && isValidDataFim && isValidIntervalDates) {

				ScreenService.confirmMessage(
					"O tempo para geração do relatório Registro PAF pode demorar até 6 minutos para ser concluído variando de acordo com o volume de informações do banco de dados. Enquanto não for possível obter o retorno da geração o sistema ficará bloqueado.<br><br>Deseja gerar o relatório mesmo assim? ",
					"question",
					function () {
			        	ScreenService.showLoader();
						FiscalFunctionsService.registroPaf(operatorData.CDFILIAL, operatorData.CDCAIXA, dataInicio, dataFim).then(function(response) {
							ScreenService.hideLoader();
							if(!_.isEmpty(response)) {
								ScreenService.showMessage(response[0].message);
								if(!response[0].error && response[0].data === 0) {
									self.handleBack(widget);
								}
							} else {
								ScreenService.showMessage("Erro desconhecido ao tentar gerar o relatório.<br> Tente novamente.");
							}
						});
						ScreenService.hideLoader();
					},
					function () { }
				);

			} else {
				var errMsg = "";
				errMsg += !isValidDataInicio ? 'inicial' : '';
				errMsg += (!isValidDataInicio && !isValidDataFim) ? ' e ' : '';
				errMsg += !isValidDataFim ? 'final' : '';

				ScreenService.showMessage('Data ' + errMsg + ' inválida(s) para a geração do relatório. Verifique os campos preenchidos e tente novamente.');
			}

		});
	};

	this.generateReportRequisicaoExterna = function (widget) {
		OperatorRepository.findOne().then(function(operatorData){

			var isValidDataInicio = self.validateDate(widget.getField('initialDate'));
			var isValidDataFim = self.validateDate(widget.getField('finalDate'));
        	var dataInicio = widget.getField('initialDate').value();
        	var dataFim = widget.getField('finalDate').value();
        	var isValidIntervalDates = self.validateIntervalDates(dataInicio, dataFim);


			if (!widget.isValid()){
	            ScreenService.showMessage("É necessário preencher todos os campos para gerar o relatório.");
	        } else if(isValidDataInicio && isValidDataFim && isValidIntervalDates) {

				ScreenService.showLoader();
				FiscalFunctionsService.requisicaoExterna(operatorData.CDFILIAL, dataInicio, dataFim).then(function(response) {
					ScreenService.hideLoader();
					if(!_.isEmpty(response)) {
						ScreenService.showMessage(response[0].message);
						if(!response[0].error && response[0].data === 0) {
							self.handleBack(widget);
						}
					} else {
						ScreenService.showMessage("Erro desconhecido ao tentar gerar o relatório.<br> Tente novamente.");
					}
				});
				ScreenService.hideLoader();

			} else {
				var errMsg = "";
				errMsg += !isValidDataInicio ? 'inicial' : '';
				errMsg += (!isValidDataInicio && !isValidDataFim) ? ' e ' : '';
				errMsg += !isValidDataFim ? 'final' : '';

				ScreenService.showMessage('Data ' + errMsg + ' inválida(s) para a geração do relatório. Verifique os campos preenchidos e tente novamente.');
			}

		});
	};

	this.handleBack = function (widget) {
		widget.fields.forEach(function (field) {
			field.clearValue();
		});

		ScreenService.closePopup();
	};

}


Configuration(function(ContextRegister){
	ContextRegister.register('FiscalFunctions', FiscalFunctions);
});